﻿;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;			
;|																															|;	
;| Title:				UB2D - 009_EngineControl												|;
;| Copyright: 	UnionBytes																			|;
;|																															|;
;| Date:				07.11.2018																			|;
;|																															|;
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;


;- Engine
XIncludeFile "Includes\UB2D.pbi" 
If Not InitUnionBytes2DEngine(#UB2D_ENGINE_DEBUG)
	Debug "UB2D initialization failed!"
	End
EndIf

UB2D_OpenScreen(1280, 720, 32, "Union Bytes 2D Engine - EngineControl", #PB_Window_ScreenCentered|#PB_Window_SystemMenu, #False)


;- Variables
Global X.i, Y.i


;- Fonts
; Loading a Typeface Font with the ID 0. (The Editor comes also With the engine.)
UB2D_LoadFont(0, "Data\Fonts\Arial_20.tf") 


;- Textures
; Loading the texture for our objects.
UB2D_LoadTexture(0, "Data\Textures\UB2D_B.png")	; Base color (albedo map)
UB2D_LoadTexture(1, "Data\Textures\UB2D_N.png") ; Normal map
UB2D_LoadTexture(2, "Data\Textures\UB2D_REM.png") ; REM map (Red = Roughness, Green = Emission, Blue = Metallic)

UB2D_LoadTexture(3, "Data\Textures\Floor_B.jpg"); Base color (albedo map)
UB2D_LoadTexture(4, "Data\Textures\Floor_N.jpg") ; Normal map
UB2D_LoadTexture(5, "Data\Textures\Floor_REM.jpg") ; REM map (Red = Roughness, Green = Emission, Blue = Metallic)


;- Materials
; Creating of a material for the UB2D Logo, with the default object PBR shader!
UB2D_CreateMaterial(0, UB2D_DefaultShaderID(#UB2D_SHADER_OBJ_PBR), 0)
UB2D_SetMaterialTexture(0, 1, 1) ; Set the normal map to index 1
UB2D_SetMaterialTexture(0, 2, 2) ; Set the rme map to index 2
UB2D_SetMaterialRoughness(0, 1.0) ; Set the roughness map multiplier
UB2D_SetMaterialMetallic(0, 1.0) ; Set the metallic map multiplier
UB2D_SetMaterialEmission(0, 0.3) ; Set the emission map multiplier
UB2D_SetMaterialSpecular(0, 0.8) ; Scale the specular amount

; Floor material, with the default object PBR shader!
UB2D_CreateMaterial(1, UB2D_DefaultShaderID(#UB2D_SHADER_OBJ_PBR), 3)
UB2D_SetMaterialTexture(1, 1, 4) ; Set the normal map to index 1
UB2D_SetMaterialTexture(1, 2, 5) ; Set the rme map to index 2
UB2D_SetMaterialRoughness(1, 1.0) ; Set the roughness map multiplier
UB2D_SetMaterialMetallic(1, 1.0) ; Set the metallic map multiplier
UB2D_SetMaterialEmission(1, 0.0) ; Set the emission map multiplier
UB2D_SetMaterialSpecular(1, 0.8) ; Scale the specular amount


;- Objects
; Creating the objects, with the material 0
UB2D_CreateObject(0, 0, 0.5, 0.5)
UB2D_AddObjectInstance(0, 0, 0.0, 0.0, 24.0)

; Creating the floor objects
UB2D_CreateObject(1, 1, 0.5, 0.5)
UB2D_ResizeObject(1, 320, 320)
For Y = 0 To 2
	For X = 0 To 3
		UB2D_AddObjectInstance(1+(Y*5+X), 1, (X-1.5)*320.0, (Y-1)*320.0, 0.0)
	Next
Next


;- World light settings
; Reducing of the direct light and the ambientlight to see the point lights effects better
UB2D_SetDirectLightIntensity(0.00)
UB2D_SetAmbientLightColor($FFFFFFFF)


;- Lights
; Create the Lights and attatching it to the UB2D Logo
UB2D_CreateLight(0, -120.0, 180.0, 260.0, 400.0, $FFFFFFFF)
UB2D_SetLightIntensity(0, 0.6)
UB2D_Attach(UB2D_ObjectInstanceID(0), UB2D_LightID(0))

UB2D_CreateLight(1, -120.0, -120.0, 260.0, 400.0, $FFFFFFFF)
UB2D_SetLightIntensity(1, 0.6)
UB2D_Attach(UB2D_ObjectInstanceID(0), UB2D_LightID(1))

UB2D_CreateLight(2, 160.0, 0.0, 320.0, 400.0, $FF04DA14)
UB2D_SetLightIntensity(2, 0.8)
UB2D_Attach(UB2D_ObjectInstanceID(0), UB2D_LightID(2))


;- Main Loop
Repeat
	
	; Normal pure basic window event handling.
	Repeat
		Select UB2D_WindowEvent()
			Case #PB_Event_CloseWindow
				Break 2
			Case #PB_Event_None
				Break
		EndSelect 
	ForEver
	
	; UB2D GUI event handling. (Needed for Engine Control)
	Select UB2D_Event()
		Case #UB2D_EVENT_GADGET
			Select UB2D_EventGadget()
			EndSelect
	EndSelect
	
	; Update of the control functions. 
	UB2D_ExamineMouse() ; (Needed For Engine Control)
	UB2D_ExamineKeyboard()
	
	; F1 open the Engine Control window.
	If UB2D_KeyboardReleased(#UB2D_KEY_F1)
		If UB2D_EngineControlStatus()
			UB2D_CloseEngineControl()
		Else
			UB2D_OpenEngineControl(0, 0)
		EndIf
	EndIf
	
	; Rotate the mesh instance
 	UB2D_RotateObjectInstance(0, 15.0*UB2D_TimeFactor())
	
	; We have to replace now the funktion UB2D_BindFrameBuffer(#UB2D_FRAMEBUFFER_FINAL, #True, #True)
	; With the following one. After the scene is rendered, the final framebuffer will be set automatically
 	UB2D_RenderScene()
 	
	; Display info text on the screen.
 	UB2D_DisplayText(0, 20, 20, "F1: Open Engine Control!", $FFFFFFFF) 	
 	
 	; Display of all GUI elements.; (Needed For Engine Control)
	UB2D_DisplayGUI()
 	
	; Flip the buffers and draw it on the window.
	UB2D_FlipBuffers()
	
	; F12 makes a screenshot
	If UB2D_KeyboardReleased(#UB2D_KEY_F12)
		UB2D_ScreenShot("009_EngineControl.jpg")
	EndIf
	
ForEver

; Free the engine and all its resources.
UB2D_Free()
End

; IDE Options = PureBasic 5.62 (Windows - x64)
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 921
; EnableBuildCount = 0